from .imports import *
from .functions import *
from .metadata import *
from .pagedata import build_page_json
from .filter_utils import extract_video_fields,get_values,get_dict_example,STRINGS_VARS
from .videoDownloader import *



from .imports import *
def safe_load_json(path):
    try:
        with open(path, "r", encoding="utf-8") as f:
            return json.load(f)
    except Exception:
        return {}

def safe_load_text(path):
    try:
        with open(path, "r", encoding="utf-8") as f:
            return f.read()
    except Exception:
        return ""

def classify_category(keywords, title="", description=""):
    kws = [k.lower() for k in keywords]
    text = " ".join(kws + [title.lower(), description.lower()])

    if any(x in text for x in ["comedy", "funny", "skit", "humor"]):
        return "Comedy"
    if any(x in text for x in ["music", "song", "album", "concert"]):
        return "Music"
    if any(x in text for x in ["news", "politic", "debate", "report"]):
        return "News & Politics"
    if any(x in text for x in ["education", "tutorial", "lesson", "howto"]):
        return "Education"
    if any(x in text for x in ["game", "gaming", "esport", "playthrough"]):
        return "Gaming"
    if any(x in text for x in ["sports", "match", "tournament"]):
        return "Sports"
    if any(x in text for x in ["tech", "review", "unboxing", "product"]):
        return "Science & Technology"
    if any(x in text for x in ["travel", "tour", "journey"]):
        return "Travel"
    if any(x in text for x in ["food", "cook", "recipe", "restaurant"]):
        return "Food"
    if any(x in text for x in ["film", "movie", "animation", "cinema"]):
        return "Film & Animation"

    return "Entertainment"

def aggregate_metadata(base_dir):
    # === Load sources ===
    info         = safe_load_json(os.path.join(base_dir, "video_info.json"))
    metadata     = safe_load_json(os.path.join(base_dir, "video_metadata.json"))
    whisper      = safe_load_json(os.path.join(base_dir, "whisper_result.json"))
    total_info   = safe_load_json(os.path.join(base_dir, "total_info.json"))
    thumbnails   = safe_load_json(os.path.join(base_dir, "thumbnails.json"))
    captions_txt = safe_load_text(os.path.join(base_dir, "captions.srt"))

    # === Collect raw fields ===
    title        = metadata.get("title") or info.get("title")
    description  = metadata.get("description") or metadata.get("summary") or info.get("description")
    keywords     = set()
    for src in (metadata, info):
        for k in ["keywords","tags","categories"]:
            if src.get(k):
                if isinstance(src[k], list):
                    keywords.update(src[k])
                else:
                    keywords.add(str(src[k]))

    # === Transcript / Captions ===
    transcript = []
    if whisper.get("text"):
        transcript.append(whisper["text"])
    if captions_txt:
        transcript.append(captions_txt)
    transcript_text = "\n".join(transcript).strip()

    # === Thumbnails ===
    thumb_candidates = []
    if metadata.get("thumbnail_url"):
        thumb_candidates.append(metadata["thumbnail_url"])
    if metadata.get("seodata",{}).get("seo_data",{}).get("thumbnail",{}).get("file_path"):
        thumb_candidates.append(metadata["seodata"]["seo_data"]["thumbnail"]["file_path"])
    if thumbnails.get("thumbnail_paths"):
        thumb_candidates.extend(thumbnails["thumbnail_paths"])

    # === Duration ===
    duration = (
        metadata.get("duration_formatted")
        or metadata.get("duration_seconds")
        or info.get("duration")
    )

    # === Category ===
    category = (
        metadata.get("category")
        or classify_category(keywords, title=title or "", description=description or "")
    )

    # === Build unified object ===
    aggregated = {
        "id": info.get("id"),
        "title": title,
        "description": description,
        "keywords": sorted(list(keywords)),
        "category": category,
        "url": info.get("webpage_url") or metadata.get("canonical_url"),
        "duration": duration,
        "uploader": (
            metadata.get("seodata",{}).get("seo_data",{}).get("uploader",{}).get("name")
            or info.get("uploader")
        ),
        "publication_date": metadata.get("seodata",{}).get("seo_data",{}).get("publication_date"),
        "video_path": total_info.get("video_path"),
        "audio_path": total_info.get("audio_path"),
        "transcript": transcript_text,
        "thumbnails": thumb_candidates,
        "schema_markup": metadata.get("seodata",{}).get("seo_data",{}).get("schema_markup"),
        "social_metadata": metadata.get("seodata",{}).get("seo_data",{}).get("social_metadata"),
        "video_metadata": metadata.get("seodata",{}).get("seo_data",{}).get("video_metadata"),
        "info_flags": total_info,
    }
    file_path = os.path.join(base_dir,"aggregated_metadata.json")
    data = json.dumps(aggregated, indent=2)
    safe_dump_to_json(data=data,file_path=file_path)
    return file_path,aggregated
def score_segments(segments, keywords):
    scored = []
    for seg in segments:
        text = seg["text"].lower()
        score = 0
        # Keyword hits
        score += sum(1 for k in keywords if k in text)
        # Sentiment/emphasis heuristics
        if "!" in text or "oh my god" in text: score += 2
        if "let's go" in text or "we can" in text: score += 2
        # Length penalty/bonus
        duration = seg["end"] - seg["start"]
        if 10 <= duration <= 45:
            score += 1
        scored.append((score, seg))
    scored.sort(key=lambda x: x[0], reverse=True)
    return scored

    






# src/db.py

# src/db.py
from .imports import *
from sqlalchemy.exc import OperationalError


ENV_VARS = {
    "dbname": "abstract_base",
    "user": "admin",
}
def sanitize_output(record: dict) -> dict:
    if not record:
        return record
    if "audio" in record:
        record["audio"] = (
            f"<{len(record['audio'])} bytes>"
            if record["audio"]
            else None
        )
    return record
def sanitize_output(record: dict) -> dict:
    if "audio" in record:
        record["audio"] = f"<{len(record['audio'])} bytes>" if record["audio"] else None
    return record
class abstractHugpyDbManager(metaclass=SingletonMeta):
    def __init__(self,*args,**kwargs):
        if not hasattr(self,'initialized'):
            self.initialized = True
            self.metadata = MetaData()
            self.DATABASE_URL = self.get_database_url(**kwargs)
            self.VIDEOSTABLE = self.get_video_stable()
            
            self.engine = self.start_engine(self.DATABASE_URL,*args,**kwargs)
    def getEngine(self):
        return self.engine if self.engine is not None else None
    def getVideosTable(self):
        return self.VIDEOSTABLE if self.VIDEOSTABLE is not None else None
    def get_database_url(self,
                         **kwargs):
        ENV_VARS.update(**kwargs)
        env_js = get_db_vars(**ENV_VARS)
        return env_js.get("dburl")

    def start_engine(self,
                    url,*args,
                    future=None,
                    pool_size=None,
                    max_overflow=None,
                    **kwargs
        ):
        future= (False if (future == False) else True)
        pool_size=poolsize if is_number(pool_size) else 10
        max_overflow= max_overflow if is_number(max_overflow) else 20
        self.engine = None
        try:

            self.engine = create_engine(
                url,
                future=future,
                pool_size=pool_size,
                max_overflow=max_overflow,
            )
        except Exception as e:
            logger.info(f"{e}")
        return self.engine

    def get_video_stable(self):
        self.VIDEOSTABLE = None
        try:
            self.VIDEOSTABLE = Table(
                "videos",
                self.metadata,
                Column("id", Integer, primary_key=True),
                Column("video_id", String, unique=True, nullable=False),
                Column("info", JSONB),
                Column("extractfields", JSONB),
                Column("rawdata", JSONB),
                Column("metadata", JSONB),
                Column("whisper", JSONB),
                Column("captions", JSONB),
                Column("thumbnails", JSONB),
                Column("total_info", JSONB),
                Column("aggregated", JSONB),
                Column("seodata", JSONB),
                Column("audio_path", String),
                Column("audio_format", String),
                Column("metatags", JSONB),
                Column("pagedata", JSONB),
                Column("created_at", TIMESTAMP, server_default=text("NOW()")),
                Column("updated_at", TIMESTAMP, server_default=text("NOW()")),
            )
        except Exception as e:
            logger.info(f"{e}")
        return self.VIDEOSTABLE

    def init_db(self,**kwargs):

        if self.engine is not None:
            try:
                self.metadata.create_all(engine)
            except Exception as e:
                logger.info(f"{e}")
        return self.engine,self.VIDEOSTABLE



    def get_video_record(self,video_id, hide_audio: bool = True):
        video_id = str(video_id)

        with self.engine.begin() as conn:
            row = conn.execute(
                select(self.VIDEOSTABLE)
                .where(self.VIDEOSTABLE.c.video_id == video_id)
            ).first()

        if not row:
            return None

        record = dict(row._mapping)
        return sanitize_output(record) if hide_audio else record




    def upsert_video(
        self,
        *args,
        video_id=None,
        **fields
    ):
        """
        Safe upsert:
        - Accepts video_id positionally or by keyword
        - If multiple values exist, first non-null wins
        """

        # -------------------------
        # Resolve video_id safely
        # -------------------------
        candidates = []

        # positional video_id
        if args:
            candidates.append(args[0])

        # keyword video_id
        if video_id is not None:
            candidates.append(video_id)

        # alias fallbacks (optional but recommended)
        for alias in ("videoId", "id"):
            val = fields.pop(alias, None)
            if val is not None:
                candidates.append(val)

        video_id = next((v for v in candidates if v is not None), None)

        if video_id is None:
            raise ValueError("upsert_video requires a non-null video_id")

        # ensure it cannot leak forward
        fields.pop("video_id", None)

        # -------------------------
        # Make fields JSON-safe
        # -------------------------
        safe_fields = {
            k: to_json_safe(v)
            for k, v in fields.items()
        }

        # -------------------------
        # Build upsert statement
        # -------------------------
        stmt = insert(self.VIDEOSTABLE).values(
            video_id=video_id,
            **safe_fields
        )

        stmt = stmt.on_conflict_do_update(
            index_elements=["video_id"],
            set_={**safe_fields, "updated_at": text("NOW()")}
        )

        with self.engine.begin() as conn:
            conn.execute(stmt)



##    def get_video_record(self,video_id: str, hide_audio: bool = True):
##        with self.engine.begin() as conn:
##            row = conn.execute(select(self.VIDEOSTABLE).where(self.VIDEOSTABLE.c.video_id == video_id)).first()
##            if not row:
##                return None
##            record = dict(row._mapping)
##            if hide_audio and "audio" in record:
##                # Replace huge binary blob with a short placeholder
##                record["audio"] = f"<{len(record['audio'])} bytes>" if record["audio"] else None
##            return record

def getHugpyDbManager(*args,**kwargs):
    return abstractHugpyDbManager(*args,**kwargs)
def getHugpyDbEngine(*args,**kwargs):     
    hugpyDb_mgr = getHugpyDbManager(*args,**kwargs)
    return hugpyDb_mgr.getEngine()
def getHugpyDbEVideosTable(*args,**kwargs):     
    hugpyDb_mgr = getHugpyDbManager(*args,**kwargs)
    return hugpyDb_mgr.getVideosTable()


def is_db_available(engine) -> bool:
    if engine is None:
        return False
    try:
        with engine.connect() as conn:
            return True
    except OperationalError:
        return False
        
        
    



from ..imports import *
from urllib.parse import urljoin
import json


def fs_path_to_url(path: str | None, base_url: str='https://clownworld.biz) -> str | None:
    """
    Convert a filesystem-style path (/var/www/..., /media/..., /...) to a public URL.
    Tunable to your nginx layout; this version assumes /var/www/* is web root.
    """
    if not path:
        return None

    # Already a URL? leave it alone
    if path.startswith("http://") or path.startswith("https://"):
        return path

    base_url = base_url.rstrip("/")
    # Map /var/www/<stuff> → https://domain/<stuff>
    if path.startswith("/var/www/media/DATA"):
        rel = path[len("/var/www//media/DATA"):]
        return f"{base_url}/{rel.lstrip('/')}"
    # Map /var/www/<stuff> → https://domain/<stuff>
    if path.startswith("/mnt/24T/media/DATA"):
        rel = path[len("/mnt/24T/media/DATA"):]
        return f"{base_url}/{rel.lstrip('/')}"

    # Site-root relative path (/media/..., /imgs/..., etc.)
    if path.startswith("/"):
        return f"{base_url}{path}"

    # Fallback – join relative against base
    return urljoin(base_url + "/", path)

def build_meta_from_video_result(result: dict,
                                 base_url: str = "https://clownworld.biz") -> dict:
    """
    Normalize your video pipeline result into the `meta` structure used by generate_meta_tags.
    """

    seo_data = (result.get("seodata") or {}).get("seo_data") or {}
    meta_data = result.get("metadata") or {}
    info_block = result.get("info") or {}

    # Video ID
    video_id = info_block.get("video_id") or (info_block.get("info") or {}).get("video_id")

    # Canonical URL
    canonical_raw = seo_data.get("canonical_url")
    if canonical_raw:
        # canonical_url in your data is "clownworld.biz" (no scheme / path)
        if not canonical_raw.startswith(("http://", "https://")):
            canonical = f"https://{canonical_raw.rstrip('/')}"
    else:
        canonical = base_url

    # If you want per-video canonicals like /?video_id=..., wire it here:
    if video_id:
        canonical = f"{canonical.rstrip('/')}/?video_id={video_id}"

    # Title / description
    seo_title = seo_data.get("seo_title") or meta_data.get("title") \
        or (video_id and f"{video_id} - Clown World") \
        or "Clown World"

    seo_description = seo_data.get("seo_description") or meta_data.get("summary")

    # Fallback to first chunk of Whisper text if summary missing
    if not seo_description and result.get("whisper", {}).get("text"):
        whisper_text = result["whisper"]["text"].strip()
        if whisper_text:
            seo_description = whisper_text[:300] + ("..." if len(whisper_text) > 300 else "")

    # Keywords
    tags_from_seo = seo_data.get("seo_tags") or []
    tags_from_meta = meta_data.get("keywords") or []
    keywords_list: list[str] = []
    for k in tags_from_seo + tags_from_meta:
        if k and k not in keywords_list:
            keywords_list.append(k)
    keywords_str = ", ".join(keywords_list)

    # Thumbnail
    thumb_obj = seo_data.get("thumbnail") or {}
    thumb_path = thumb_obj.get("file_path")

    if not thumb_path:
        # fall back to first thumbnail path in the list
        thumbs_paths = ((result.get("thumbnails") or {}).get("paths") or [])
        thumb_path = thumbs_paths[0] if thumbs_paths else None

    thumb_url = fs_path_to_url(thumb_path, base_url) if thumb_path else None

    # Video path / URL
    schema_markup = seo_data.get("schema_markup") or {}
    video_fs_path = (
        schema_markup.get("contentUrl")
        or (info_block.get("schema_paths") or {}).get("video_path")
        or info_block.get("video_path")
    )
    video_url = fs_path_to_url(video_fs_path, base_url) if video_fs_path else None

    # Social meta (og:* and twitter:*)
    social = seo_data.get("social_metadata") or {}

    og = {
        "title": social.get("og:title") or seo_title,
        "description": social.get("og:description") or seo_description,
        "url": canonical,
        "image": fs_path_to_url(social.get("og:image"), base_url)
                 if social.get("og:image") else thumb_url,
        "image_alt": thumb_obj.get("alt_text"),
        "image_width": "1200",
        "image_height": "627",
        "image_type": "image/jpeg",
        "type": "video.other",
        "site_name": "Clown World",
        "locale": "en_US",
        "video": fs_path_to_url(social.get("og:video"), base_url)
                 if social.get("og:video") else video_url,
    }

    # Strip empty keys so generate_meta_tags' conditionals behave nicely
    og = {k: v for k, v in og.items() if v}

    twitter = {
        "card": social.get("twitter:card", "player"),
        "title": social.get("twitter:title") or seo_title,
        "description": social.get("twitter:description") or seo_description,
        "image": fs_path_to_url(social.get("twitter:image"), base_url)
                 if social.get("twitter:image") else thumb_url,
        "site": "@clownworld",
        "creator": "@clownworld",
        "domain": base_url.replace("https://", "").replace("http://", ""),
        # optional: if you later have a dedicated player URL, set it here:
        # "player": f"{base_url.rstrip('/')}/player/{video_id}" if video_id else None,
    }
    twitter = {k: v for k, v in twitter.items() if v}

    # "Other" generic tags
    uploader = (seo_data.get("uploader") or {}).get("name") or "Clown World Team"
    category = seo_data.get("category") or meta_data.get("category") or "General"

    other = {
        "robots": "index, follow",
        "googlebot": "index, follow",
        "bingbot": "noarchive",
        "yahooContent": "article",
        "author": uploader,
        "revisit-after": "7 days",
        "rating": category,
        "distribution": "global",
        "viewport": "width=device-width, initial-scale=1",
        "application-name": "Clown World",
        "theme_color": "#000000",
        "color_scheme": "light dark",
        "charset": "utf-8",
        "content_type": "text/html; charset=utf-8",
    }

    # Final meta dict in your existing shape
    meta = {
        "title": seo_title,
        "description": seo_description,
        "description_html": None,
        "keywords": keywords_str,
        "thumbnail": thumb_url,
        "thumbnail_resized_link": thumb_url,  # so your existing logic finds it
        "canonical": canonical,
        "variants": [canonical],
        "og": og,
        "twitter": twitter,
        "other": other,
        "schema_markup": schema_markup,
    }
    return meta

def generate_meta_tags(meta, base_url=None, json_path=None, **kwargs):
    base_url = base_url or (meta.get('variants') or [None])[0] or ""
    tags = []
    json_path = (json_path or "").split('json_pages/')[-1]

    # ---------------- Base tags ----------------
    title = meta.get("title") or "clownworld.biz"
    description = (
        meta.get("description_html")
        or meta.get("description")
        or "Explore content from clownworld.biz."
    )
    keywords = meta.get("keywords", "")

    tags.append(f"<title>{title}</title>")
    tags.append(f'<meta name="description" content="{description}" />')
    tags.append(f'<meta name="keywords" content="{keywords}" />')

    # Favicon
    favicon = (
        meta.get("thumbnail_resized_link")
        or (meta.get("og", {}).get("image") if meta.get("og") else None)
        or meta.get("thumbnail_link")
        or meta.get("thumbnail_resized")
        or meta.get("thumbnail")
        or "/imgs/favicon.ico"
    )
    if favicon and base_url:
        favicon = fs_path_to_url(favicon, base_url)
    tags.append(f'<link rel="icon" href="{favicon}" type="image/x-icon" />')

    # ---------------- Universal crawler tags ----------------
    other = meta.get("other", {}) or {}
    tags.append(f'<meta name="robots" content="{other.get("robots", "index, follow")}" />')
    tags.append(f'<meta name="googlebot" content="{other.get("googlebot", "index, follow")}" />')
    tags.append(f'<meta name="bingbot" content="{other.get("bingbot", "noarchive")}" />')
    tags.append(f'<meta name="yahooContent" content="{other.get("yahooContent", "article")}" />')
    tags.append(f'<meta name="author" content="{other.get("author", "Clown World Team")}" />')
    tags.append(f'<meta name="revisit-after" content="{other.get("revisit-after", "7 days")}" />')
    tags.append(f'<meta name="rating" content="{other.get("rating", "General")}" />')
    tags.append(f'<meta name="distribution" content="{other.get("distribution", "global")}" />')

    if other.get("msvalidate.01"):
        tags.append(f'<meta name="msvalidate.01" content="{other["msvalidate.01"]}" />')
    if other.get("yandex-verification"):
        tags.append(f'<meta name="yandex-verification" content="{other["yandex-verification"]}" />')

    # ---------------- Open Graph ----------------
    og = meta.get("og", {}) or {}

    og_title = og.get("title", title)
    og_description = og.get("description", description)

    # URL: prefer explicit og:url, then canonical, then base_url + json_path
    og_url = og.get("url") or meta.get("canonical") or (
        f"{base_url.rstrip('/')}{json_path}" if base_url else None
    )
    if og_url and base_url:
        og_url = fs_path_to_url(og_url, base_url)

    og_image = (
        meta.get("thumbnail_resized_link")
        or og.get("image")
        or meta.get("thumbnail_link")
        or meta.get("thumbnail_resized")
        or meta.get("thumbnail")
    )
    if og_image and base_url:
        og_image = fs_path_to_url(og_image, base_url)

    tags.append(f'<meta property="og:title" content="{og_title}" />')
    tags.append(f'<meta property="og:description" content="{og_description}" />')
    if og_url:
        tags.append(f'<meta property="og:url" content="{og_url}" />')
    if og_image:
        tags.append(f'<meta property="og:image" content="{og_image}" />')

    if og.get("image_alt"):
        tags.append(f'<meta property="og:image:alt" content="{og["image_alt"]}" />')
    tags.append(f'<meta property="og:image:width" content="{og.get("image_width", "1200")}" />')
    tags.append(f'<meta property="og:image:height" content="{og.get("image_height", "627")}" />')
    tags.append(f'<meta property="og:image:type" content="{og.get("image_type", "image/jpeg")}" />')
    tags.append(f'<meta property="og:type" content="{og.get("type", "article")}" />')
    if og.get("site_name"):
        tags.append(f'<meta property="og:site_name" content="{og["site_name"]}" />')
    tags.append(f'<meta property="og:locale" content="{og.get("locale", "en_US")}" />')

    # Video-specific OG tags
    if og.get("video"):
        og_video = fs_path_to_url(og["video"], base_url) if base_url else og["video"]
        tags.append(f'<meta property="og:video" content="{og_video}" />')
        if og.get("video_type"):
            tags.append(f'<meta property="og:video:type" content="{og["video_type"]}" />')
        if og.get("video_width"):
            tags.append(f'<meta property="og:video:width" content="{og["video_width"]}" />')
        if og.get("video_height"):
            tags.append(f'<meta property="og:video:height" content="{og["video_height"]}" />')

    if og.get("fb_app_id"):
        tags.append(f'<meta property="fb:app_id" content="{og["fb_app_id"]}" />')
    if og.get("updated_time"):
        tags.append(f'<meta property="og:updated_time" content="{og["updated_time"]}" />')
    if og.get("article"):
        article = og["article"]
        if article.get("published_time"):
            tags.append(f'<meta property="article:published_time" content="{article["published_time"]}" />')
        if article.get("modified_time"):
            tags.append(f'<meta property="article:modified_time" content="{article["modified_time"]}" />')
        if article.get("section"):
            tags.append(f'<meta property="article:section" content="{article["section"]}" />')
        if article.get("tag"):
            for tag in article["tag"]:
                tags.append(f'<meta property="article:tag" content="{tag}" />')

    # ---------------- Twitter Cards ----------------
    twitter = meta.get("twitter", {}) or {}
    tags.append(f'<meta name="twitter:card" content="{twitter.get("card", "summary_large_image")}" />')
    tags.append(f'<meta name="twitter:title" content="{twitter.get("title", title)}" />')
    tags.append(f'<meta name="twitter:description" content="{twitter.get("description", description)}" />')

    tw_image = twitter.get("image") or meta.get("thumbnail") or og_image
    if tw_image and base_url:
        tw_image = fs_path_to_url(tw_image, base_url)
    if tw_image:
        tags.append(f'<meta name="twitter:image" content="{tw_image}" />')

    if twitter.get("image_alt"):
        tags.append(f'<meta name="twitter:image:alt" content="{twitter["image_alt"]}" />')
    tags.append(f'<meta name="twitter:image:type" content="{twitter.get("image_type", "image/jpeg")}" />')
    tags.append(f'<meta name="twitter:site" content="{twitter.get("site", "@clownworld")}" />')
    if twitter.get("site:id"):
        tags.append(f'<meta name="twitter:site:id" content="{twitter["site:id"]}" />')
    tags.append(f'<meta name="twitter:creator" content="{twitter.get("creator", "@clownworld")}" />')
    if twitter.get("creator:id"):
        tags.append(f'<meta name="twitter:creator:id" content="{twitter["creator:id"]}" />')
    tags.append(f'<meta name="twitter:domain" content="{twitter.get("domain", "clownworld.biz")}" />')

    # Optional: player tags if you eventually add them
    if twitter.get("player"):
        tags.append(f'<meta name="twitter:player" content="{twitter["player"]}" />')
    if twitter.get("player:width"):
        tags.append(f'<meta name="twitter:player:width" content="{twitter["player:width"]}" />')
    if twitter.get("player:height"):
        tags.append(f'<meta name="twitter:player:height" content="{twitter["player:height"]}" />')

    # ---------------- Other tags ----------------
    tags.append(f'<meta name="viewport" content="{other.get("viewport", "width=device-width, initial-scale=1")}" />')
    if other.get("application-name"):
        tags.append(f'<meta name="application-name" content="{other["application-name"]}" />')
    tags.append(f'<meta name="theme-color" content="{other.get("theme_color", "#FFFFFF")}" />')
    tags.append(f'<meta name="color-scheme" content="{other.get("color_scheme", "light")}" />')
    if other.get("charset"):
        tags.append(f'<meta charset="{other["charset"]}" />')
    if other.get("content_type"):
        tags.append(f'<meta http-equiv="content-type" content="{other["content_type"]}" />')
    if other.get("manifest"):
        tags.append(f'<link rel="manifest" href="{other["manifest"]}" />')

    canonical = meta.get("canonical") or og_url
    if canonical and base_url:
        canonical = fs_path_to_url(canonical, base_url)
    if canonical:
        tags.append(f'<link rel="canonical" href="{canonical}" />')

    # ---------------- JSON-LD Schema.org (VideoObject) ----------------
    schema_markup = meta.get("schema_markup")
    if schema_markup:
        sm = dict(schema_markup)  # shallow copy
        # normalize contentUrl & thumbnailUrl to public URLs
        for k in ("contentUrl", "thumbnailUrl"):
            if sm.get(k) and base_url:
                sm[k] = fs_path_to_url(sm[k], base_url)
        tags.append(
            '<script type="application/ld+json">'
            + json.dumps(sm, ensure_ascii=False)
            + "</script>"
        )

    return "\n".join(tags)
def build_seo_meta_from_result(result: dict, base_domain: str = "https://clownworld.biz") -> dict:
    """
    Take the huge result blob you pasted and normalize it into a clean SEO meta JSON
    that React can easily consume.
    """
    # Core references
    video_id = result["info"]["info"]["video_id"]
    seo_data = (result.get("seodata") or {}).get("seo_data") or {}
    metadata = result.get("metadata") or {}
    info = result.get("info") or {}

    # Canonical path for this video on your site
    # tweak this to match your real route: e.g. /video/<id> or /watch/<id>
    page_path = f"/video/{video_id}"
    canonical_url = f"{base_domain}{page_path}"

    # Thumbnail
    thumb = (seo_data.get("thumbnail") or {})
    thumb_path = thumb.get("file_path") or (seo_data.get("social_metadata") or {}).get("og:image")

    # OG / Twitter pulled from seo_data.social_metadata
    social = seo_data.get("social_metadata") or {}

    og = {
        "title": social.get("og:title") or seo_data.get("seo_title") or metadata.get("title"),
        "description": social.get("og:description") or seo_data.get("seo_description") or metadata.get("summary"),
        "url": canonical_url,
        "image": social.get("og:image") or thumb_path,
        "video": social.get("og:video") or (seo_data.get("schema_markup") or {}).get("contentUrl"),
        "site_name": "Clown World",
        "locale": "en_US",
        "type": "video.other",
    }

    twitter = {
        "card": social.get("twitter:card", "player"),
        "title": social.get("twitter:title") or og["title"],
        "description": social.get("twitter:description") or og["description"],
        "image": social.get("twitter:image") or og["image"],
        "site": "@clownworld",
        "creator": "@clownworld",
        "domain": "clownworld.biz",
    }

    schema_markup = seo_data.get("schema_markup") or {}
    # Ensure schema has canonical URL
    if schema_markup:
        schema_markup = {
            **schema_markup,
            "url": canonical_url,
            "embedUrl": schema_markup.get("embedUrl") or canonical_url,
        }

    other = {
        "robots": "index, follow",
        "googlebot": "index, follow",
        "bingbot": "noarchive",
        "yahooContent": "article",
        "author": (seo_data.get("uploader") or {}).get("name", "Clown World"),
        "revisit-after": "7 days",
        "rating": seo_data.get("category", "General"),
        "distribution": "global",
        "viewport": "width=device-width, initial-scale=1",
        "theme_color": "#000000",
        "color_scheme": "dark light",
    }

    seo_meta = {
        "video_id": video_id,
        "slug": video_id,  # you can swap for a human slug later
        "path": page_path,
        "canonical": canonical_url,
        "title": seo_data.get("seo_title") or metadata.get("title"),
        "description": seo_data.get("seo_description") or metadata.get("summary"),
        "keywords": seo_data.get("seo_tags") or metadata.get("keywords") or [],
        "duration_seconds": seo_data.get("duration_seconds"),
        "thumbnail": {
            "src": thumb_path,
            "alt": thumb.get("alt_text", video_id),
        },
        "og": og,
        "twitter": twitter,
        "schema": schema_markup,
        "other": other,
    }
    return seo_meta

def generate_meta_tags_from_result(result: dict,
                                   base_url: str = "https://clownworld.biz",
                                   json_path: str | None = None,
                                   **kwargs) -> str:
    meta = build_meta_from_video_result(result, base_url=base_url)
    return generate_meta_tags(meta, base_url=base_url, json_path=json_path, **kwargs)



from ..imports import *
logger = get_logFile(__name__)

def resolve_model_path(entry):
    """Return a valid model path or HF repo id from DEFAULT_PATHS entry."""
    if entry is None:
        logger.error("{entry}: DEFAULT_PATHS entry missing.")
        return None
    if isinstance(entry,str) and entry in MODULE_DEFAULTS:
       return MODULE_DEFAULTS.get(entry)

    if isinstance(entry, dict):
        local_path = entry.get("path")
        repo_id = entry.get("id")
        name = entry.get("name")

        if local_path and os.path.exists(local_path):
            logger.info(f"{name} resolved local model path: {local_path}")
            return local_path

        if repo_id:
            logger.info(f"{name} resolved remote repo id: {repo_id}")
            return repo_id

        logger.error(f"{name}: malformed entry: {entry}")
        return None

    if isinstance(entry, str):
        logger.info(f"{name} using direct model string: {entry}")
        return entry

    logger.error(f"{name}: invalid model path type: {type(entry)}")
    return None



# src/pipeline_ur.py
from .imports import *

def get_thumbnails(video_id: str, video_path: str, thumbnails_path: str):
    """
    Extract thumbnails + OCR → persist into DB JSON.
    """
##    print(f"video_id=={video_id}\nvideo_path=={video_path}\nthumbnails_path={thumbnails_path}")
    paths = extract_video_frames_unique(video_path, thumbnails_path, video_id=video_id)
    texts = [ocr_image(p) for p in paths]
    thumbnails = {"paths": paths, "texts": texts}
    upsert_video(video_id, thumbnails=thumbnails)
    return thumbnails

def get_all_metadata(video_id,data):
    base_url = "https://clownworld.biz"
    record = get_video_record(video_id=video_id)
    video_info = record.get("info") or {}
    directory = video_info.get("directory")
    meta_file_path = os.path.join(directory, "meta.json")
    metadict = build_seo_meta_from_result(data, base_url=base_url)
    # Make sure all paths inside metadict are URL-style before saving
    metadict_public = convert_paths_to_urls(metadict, base_url=base_url)
    upsert_video(video_id, metatags=metadict_public)
    safe_dump_to_json(data=metadict_public, file_path=meta_file_path)
    return metadict_public

            
def get_pipeline_key(key):
    pipeline_key_map = {
     'all': ['all', 'all_data', 'data', 'get_all', 'get'],
     'info': ['info', 'video_info', 'video'],
     'id': ['id', 'video_id'],
     'url': ['url', 'video_url'],
     'path': ['path', 'video_path'],
     'record': ['record','video_record'],
     'dir': ['directory', 'video_directory','video_dir','dir'],
     'raw': ['raw', 'raw_data'],
     'extract': ['extract', 'extract_fields', 'fields'],
     'download': ['download', 'download_video'],
     'audio': ['audio', 'extract_audio'],
     'whisper': ['whisper'],
     'captions': ['captions'],
     'metadata': ['metadata', 'meta_data', 'meta'],
     'thumbnails': ['thumbnails'],
     'seodata': ['seodata', 'seo_data', 'seo'],
     'metatags': ['metatags', 'meta_tags', 'tags'],
     'pagedata': ['pagedata', 'page_data', 'page']
     }
    if key:
        if pipeline_key_map.get(key):
            return key
        for pipe_key,pipe_values in pipeline_key_map.items():
            if key in pipe_values:
                return pipe_key
    return "all"



from .imports import *
logger = logging.getLogger("FlanManager")

class FlanManager(BaseModelManager):
    """
    Managed interface for google/flan-t5-xl (text2text generation).
    Loads once per process, reuses shared TorchEnvManager context.
    """

    def __init__(self, model_dir=None, use_quantization=None):
        # BaseModelManager handles singleton setup and environment resolution
        super().__init__(modrl_name="flan", model_dir=model_dir, use_quantization=use_quantization)
        self.lock = threading.Lock()
    # ------------------------------------------------------------------
    # Model + tokenizer loading (overrides BaseModelManager)
    # ------------------------------------------------------------------
    def _load_model_and_tokenizer(self):
        AutoTokenizer = get_AutoTokenizer()
        AutoModelForSeq2SeqLM = get_AutoModelForSeq2SeqLM()

        self.tokenizer = AutoTokenizer.from_pretrained(self.model_dir)
        self.model = AutoModelForSeq2SeqLM.from_pretrained(self.model_dir).to(self.device)

        logger.info(f"Flan model loaded on {self.device} ({self.torch_dtype})")


    # ------------------------------------------------------------------
    # Custom summarization wrapper
    # ------------------------------------------------------------------
    def summarize(
        self,
        text: str,
        max_chunk: int = None,
        min_length: int = None,
        max_length: int = None,
        do_sample: bool = False,
    ) -> str:
        
        if not self.pipeline:
            self._safe_preload()
        max_chunk = zero_or_default(max_chunk,default=512)
        if not max_length and not min_length:
            min_length = zero_or_default(min_length,default=100)
            max_length = zero_or_default(max_length,default=512)
        elif not is_number(max_length) and is_number(min_length):
            max_length = int(min_length*(512/100))
        elif is_number(max_length) and not is_number(min_length):
            min_length = int(max_length*(100/512))
        elif is_number(max_length) and is_number(min_length) and max_length < min_length:
            min_length = int(max_length*(100/512))
        prompt = (
            "You are a highly observant assistant tasked with summarizing long, unscripted video monologues.\n\n"
            f"TEXT:\n{text}\n\n"
            "INSTRUCTIONS:\n"
            "Summarize the speaker’s core points and tone as if describing the monologue "
            "to someone who hasn’t heard it. Group related ideas together. Highlight interesting "
            "or unusual claims. Use descriptive language. Output a full narrative paragraph (or two), not bullet points."
        )

        result = self.pipeline(
            prompt,
            max_length=max_length,
            min_length=min_length,
            do_sample=do_sample,
            num_return_sequences=1,
        )[0]["generated_text"]

        return result.strip()

    # ------------------------------------------------------------------
    # Generate concise document title (PDF or raw text)
    # ------------------------------------------------------------------
    def generate_title(self, source: str | Path, from_pdf: bool = True, max_chars: int = 1200) -> str:
        """
        Generate a short, meaningful title (5–12 words) for text or PDF content.
        Uses the same Flan pipeline already loaded in this manager.
        """
        if not self.pipeline:
            self._safe_preload()

        # --- extract text ---
        if from_pdf:
            pdf_path = Path(source)
            text = ""
            try:
                with fitz.open(pdf_path) as doc:
                    for page in doc[:2]:
                        text += page.get_text("text") + "\n"
                        if len(text) > max_chars:
                            break
            except Exception as e:
                logger.error(f"PDF read failed for {pdf_path}: {e}")
                text = str(pdf_path.stem)
        else:
            text = str(source)

        text = text.strip()[:max_chars]

        # --- build title prompt ---
        prompt = (
            "You are a precise titler. "
            "Generate a clear, original, and informative title (5–12 words, no punctuation at the end) "
            "for the following document excerpt:\n\n"
            f"{text}\n\nTitle:"
        )

        # --- generate ---
        result = self.pipeline(
            prompt,
            max_length=24,
            min_length=5,
            do_sample=False,
            num_return_sequences=1,
        )[0]["generated_text"]

        # --- postprocess ---
        title = re.sub(r"^Title[:\- ]+", "", result.strip(), flags=re.I)
        title = re.sub(r"\s+", " ", title).strip(" -:;,.")
        title = title.title()

        # fallback: if it's empty or looks like the first line repeated
        if not title or title.lower() in text.lower()[:120]:
            title = text.split("\n")[0].strip().title()

        logger.info(f"Generated title: {title}")
        return title


    def get_flan_summary(
        self,
        text: str,
        max_chunk: int = None,
        min_length: int = None,
        max_length: int = None,
        do_sample: bool = None
    ) -> str:
        prompt = f"""
You are a highly observant assistant tasked with summarizing long, unscripted video monologues.

TEXT:
{text}

INSTRUCTIONS:
Summarize the speaker’s core points and tone as if describing the monologue to someone who hasn’t heard it.
Group related ideas together. Highlight interesting or unusual claims. Use descriptive language.
Output a full narrative paragraph (or two), not bullet points.
"""

        do_sample = do_sample or False
        return self.summarizer(prompt,
                          max_length=max_length,
                          min_length=min_length,
                          do_sample=do_sample)[0]['generated_text']



from .registry_functions import *
from .get_video_url_path import get_video_url_path_from_args
import inspect
import os

def _caller_summary(skip: int = 2, depth: int = 5) -> list[dict]:
    """
    Returns a compact call trace showing who called into this function.
    skip=2 skips this helper + the immediate wrapper.
    """
    frames = inspect.stack()[skip: skip + depth]
    out = []
    for f in frames:
        out.append({
            "file": os.path.basename(f.filename),
            "line": f.lineno,
            "func": f.function,
        })
    return out

class infoRegistry(metaclass=SingletonMeta):
    def __init__(self, video_root=None, documents_root=None, flat_layout: bool = False, **kwargs):
        if not hasattr(self, "initialized"):
            self.initialized = True
            self.db_mgr = getHugpyDbManager(**kwargs)
            self.engine = getHugpyDbEngine()
            
            self.videos_root = VIDEOS_ROOT_DEFAULT

    # ─────────────────────────────────────────────
    # Helper: make sure we always have a video_id
    # ─────────────────────────────────────────────
    def _normalize_video_id(self, videoId: Optional[str], data: Any) -> Optional[str]:
        """
        Ensure we end up with a non-empty video_id.

        Priority:
            1. explicit video_id argument
            2. data["video_id"]
            3. data["id"] (yt-dlp field)
        """
        if videoId:
            return str(videoId)

        if isinstance(data, dict):
            candidate = data.get("video_id") or data.get("id")
            if candidate:
                return str(candidate)

        return None

    def _upsert(self, videoId,**data):
        # 🔹 Derive a real id from the payload if needed

        videoId = self._normalize_video_id(videoId, data)

        # 🔹 Final guard: never write a NULL PK to the DB
        if not videoId:
            logger.warning(
                "infoRegistry._upsert called without a usable video_id; "
                "skipping upsert. data.id=%r",
                (data.get("id") if isinstance(data, dict) else None),
            )
            return

        # Also ensure the info blob itself carries the normalized id

        self.db_mgr.upsert_video(videoId,**data)


    def get_video_info(self, **kwargs):
        hide_audio = kwargs.get('hide_audio', True)
        force_refresh = kwargs.get('force_refresh', False)

        url, video_id, video_path, info, video_dir = get_video_url_path_from_args(**kwargs)

        # 🚨 Detect bad call immediately
        if not (url or video_id or video_path):
            logger.error(
                "❌ get_video_info called with NO identity. kwargs=%r caller=%r",
                kwargs,
                _caller_summary(),
            )
            return None
        logger.info(f"kwargs for get_video_info == {url,video_id,video_path,video_dir}")
        # ── 1) Try to derive video_id from url/path before DB / yt-dlp
        if not video_id:
            if url:
                video_id = get_video_id_from_url(url)
            elif video_path:
                video_id = generate_video_id(video_path)
        logger.info(f"kwargs2 for get_video_info == {url,video_id,video_path,video_dir}")

        # ── 2) If we *do* know video_id, see if it's already in DB
        if video_id:
            row_mapping = self.db_mgr.get_video_record(video_id=video_id,hide_audio=hide_audio)
            if row_mapping and not force_refresh:
                return row_mapping
        # ── 3) Hit yt-dlp
        info = get_yt_dlp_info(url) if url else {}
        logger.info(f"kwargs3 for get_video_info == {url,video_id,video_path,video_dir}")
        
        # If we *still* have nothing, bail out gracefully
        if not video_id:
            logger.warning(
                "get_video_info: could not determine video_id for url=%r, video_path=%r; "
                "skipping DB write.",
                url,
                video_path,
            )
            return None

        # Make sure the blob advertises the final id
        if isinstance(info, dict):
            info["video_id"] = video_id

        # Standardize paths before storage
        info = ensure_standard_paths(
            info,
            video_id=video_id,
            root_dir=self.videos_root,
            video_url=url,
            video_path=video_path,
        )
        
        if info and video_id:
            info["video_id"]= video_id
            self._upsert(video_id ,info=info)
            return info

        return None

    def edit_info(self, data, **kwargs):
        url,video_id,video_path,info,video_dir = get_video_url_path_from_args(**kwargs)

        # Keep the same normalization logic here too
        if not video_id:
            if url:
                video_id = get_video_id_from_url(url)
            elif video_path:
                video_id = generate_video_id(video_path)

        video_id = self._normalize_video_id(video_id, data)
        if not video_id:
            logger.warning(
                "edit_info called without a usable video_id; skipping edit. url=%r video_path=%r",
                url,
                video_path,
            )
            return None

        self._upsert(video_id, **data)
        return self.get_video_info(video_id=video_id, force_refresh=True)
def get_info_registry(video_root=None, documents_root=None, flat_layout: bool = False, **kwargs):
    return infoRegistry(video_root=video_root, documents_root=documents_root, flat_layout=flat_layout)
def upsert_video(video_id: str,video_root=None, documents_root=None, flat_layout: bool = False,**fields):
    info_registry = get_info_registry(video_root=video_root, documents_root=documents_root, flat_layout=flat_layout)

    return info_registry._upsert(video_id, **fields)
def get_video_record(force_refresh=False,hide_audio=True,video_root=None, documents_root=None, flat_layout: bool = False,**kwargs):
    info_registry = get_info_registry(video_root=video_root, documents_root=documents_root, flat_layout=flat_layout)
    url,video_id,video_path,info,video_dir = get_video_url_path_from_args(**kwargs)
    return info_registry.get_video_info(url=url, video_id=video_id, video_path=video_path, force_refresh=force_refresh,hide_audio=hide_audio)




from .imports import *
def _save_registry_unlocked(self):
    # Caller holds write_lock (thread-safety); now add process-safety.
    tmp = f"{self.registry_path}.tmp"
    safe_dump_to_file(self.registry, tmp)
    # atomic replace still helps, but lock ensures we don't interleave writers
    with open(self.registry_path, "wb") as f:
        portalocker.lock(f, portalocker.LOCK_EX)
        with open(tmp, "rb") as src:
            f.write(src.read())
        f.flush()
        os.fsync(f.fileno())
        portalocker.unlock(f)
    os.remove(tmp)
def get_file_data(filepath, default=None):
    if not os.path.isfile(filepath):
        os.makedirs(os.path.dirname(filepath) or ".", exist_ok=True)
        safe_dump_to_file(default or {}, filepath)
    return safe_read_from_file(file_path=filepath)

def load_json_keys(filepath, main_data=None, default=None):
    base = default or {}
    main = dict(main_data or {})
    try:
        disk = get_file_data(filepath, default=base)
        for k, v in (main or {}).items():
            if isinstance(v, dict):
                v.update(disk.get(k, {}))
            else:
                main[k] = disk.get(k, v)
        # Also merge any keys present on disk but missing in main
        for k, v in disk.items():
            main.setdefault(k, v)
        return main
    except Exception:
        return main or base



def _first_existing(*paths):
    for p in paths:
        if p and os.path.isdir(p):
            return p
    return None

def get_videos_root(explicit: str | None = None, envPath: str | None = None) -> str:
    """
    Resolve the videos root with this priority:
      1) explicit arg
      2) env file (VIDEOS_ROOT or legacy DATA_DIRECTORY)
      3) process env (VIDEOS_ROOT or legacy DATA_DIRECTORY)
      4) hard default (/mnt/24T/media/DATA/videos)
    """
    # 1) explicit
    if explicit:
        os.makedirs(explicit, exist_ok=True)
        return explicit

    # 2) env file
    v_from_envfile = (
        get_env_value(key="VIDEOS_ROOT", path=envPath)
        or get_env_value(key="DATA_DIRECTORY", path=envPath)  # legacy
    )
    if v_from_envfile:
        os.makedirs(v_from_envfile, exist_ok=True)
        return v_from_envfile

    # 3) process env
    v_from_env = os.getenv("VIDEOS_ROOT") or os.getenv("DATA_DIRECTORY")  # legacy
    if v_from_env:
        os.makedirs(v_from_env, exist_ok=True)
        return v_from_env

    # 4) hard default
    os.makedirs(DEFAULT_VIDEOS_ROOT, exist_ok=True)
    return DEFAULT_VIDEOS_ROOT


def get_documents_root(explicit: str | None = None, envPath: str | None = None) -> str:
    """
    Same idea as get_videos_root, but for documents/registry.
    Priority: explicit -> env file DOCUMENTS_ROOT -> env DOCUMENTS_ROOT -> hard default.
    """
    if explicit:
        os.makedirs(explicit, exist_ok=True)
        return explicit

    d_from_envfile = get_env_value(key="DOCUMENTS_ROOT", path=envPath)
    if d_from_envfile:
        os.makedirs(d_from_envfile, exist_ok=True)
        return d_from_envfile

    d_from_env = os.getenv("DOCUMENTS_ROOT")
    if d_from_env:
        os.makedirs(d_from_env, exist_ok=True)
        return d_from_env

    os.makedirs(DEFAULT_DOCUMENTS_ROOT, exist_ok=True)
    return DEFAULT_DOCUMENTS_ROOT


def get_video_directory(key: str | None = None, envPath: str | None = None, videos_root: str | None = None) -> str:
    """
    Assure that a valid *videos* directory exists and return its path.

    Priority:
      - videos_root arg (if passed)
      - env file (VIDEOS_ROOT or legacy DATA_DIRECTORY)
      - process env (VIDEOS_ROOT or legacy DATA_DIRECTORY)
      - DEFAULT_VIDEOS_ROOT (/mnt/24T/media/DATA/videos)
    """
    # keep key/envPath for backward compatibility with callers that used an env file
    # but prefer the explicit arg if provided
    root = get_videos_root(explicit=videos_root, envPath=envPath)
    os.makedirs(root, exist_ok=True)
    logger.info(f"using videos root: {root}")
    return root



from .bigbirdManager import *
##from .flanManager import *
##from .keybertManager import *
##from .summarizerManager import *
##from .whisperManager import *
##from .generation import *
####from .zerosearchManager import *
####from .deepcoderManager import *
##bp_list = [
####    hugpy_zerosearch_bp,
####    hugpy_deepcoder_bp,
##    whisper_bp,
##    summarizer_bp,
##    keybert_bp,
##    flan_summarizer_bp,
##    summarizer_bp,
##           ]
##
